from typing import Union

from redis import Redis
from redis.asyncio import Redis as AsyncRedis
from redis.client import bool_ok, int_or_none

from tair.tairbloom import TairBloomCommands
from tair.taircpc import CpcUpdate2judResult, TairCpcCommands
from tair.tairdoc import TairDocCommands
from tair.tairgis import TairGisCommands
from tair.tairhash import (
    TairHashCommands,
    parse_exhgetall,
    parse_exhgetwithver,
    parse_exhincrbyfloat,
    parse_exhmgetwithver,
    parse_exhscan,
)
from tair.tairroaring import TairRoaringCommands, parse_tr_scan
from tair.tairsearch import ScandocidResult, TairSearchCommands
from tair.tairstring import (
    TairStringCommands,
    parse_excas,
    parse_exget,
    parse_exincrbyfloat,
    parse_exset,
)
from tair.tairts import TairTsCommands
from tair.tairvector import (
    TairVectorCommands,
    parse_tvs_get_index_result,
    parse_tvs_get_result,
    parse_tvs_hmget_result,
    parse_tvs_msearch_result,
    parse_tvs_search_result,
)
from tair.tairzset import TairZsetCommands, parse_tair_zset_items


class TairCommands(
    TairHashCommands,
    TairStringCommands,
    TairZsetCommands,
    TairBloomCommands,
    TairRoaringCommands,
    TairSearchCommands,
    TairGisCommands,
    TairDocCommands,
    TairTsCommands,
    TairCpcCommands,
    TairVectorCommands,
):
    pass


def str_if_bytes(value: Union[str, bytes]) -> str:
    return (
        value.decode("utf-8", errors="replace") if isinstance(value, bytes) else value
    )


def bool_ok(resp) -> bool:
    return str_if_bytes(resp) == "OK"


TAIR_RESPONSE_CALLBACKS = {
    # TairString
    "EXSET": parse_exset,
    "EXGET": parse_exget,
    "EXINCRBYFLOAT": parse_exincrbyfloat,
    "EXCAS": parse_excas,
    # TairHash
    "EXHMSET": bool_ok,
    "EXHINCRBYFLOAT": parse_exhincrbyfloat,
    "EXHGETWITHVER": parse_exhgetwithver,
    "EXHMGETWITHVER": parse_exhmgetwithver,
    "EXHGETALL": parse_exhgetall,
    "EXHSCAN": parse_exhscan,
    # TairZset
    "EXZINCRBY": str_if_bytes,
    "EXZSCORE": str_if_bytes,
    "EXZRANGE": parse_tair_zset_items,
    "EXZREVRANGE": parse_tair_zset_items,
    "EXZRANGEBYSCORE": parse_tair_zset_items,
    "EXZREVRANGEBYSCORE": parse_tair_zset_items,
    # TairBloom
    "BF.RESERVE": bool_ok,
    # TairRoaring
    "TR.GETBIT": lambda resp: resp if resp == 1 else 0,
    "TR.APPENDINTARRAY": bool_ok,
    "TR.SETINTARRAY": bool_ok,
    "TR.SETBITARRAY": bool_ok,
    "TR.OPTIMIZE": bool_ok,
    "TR.SCAN": parse_tr_scan,
    "TR.RANGEBITARRAY": lambda resp: resp.decode(),
    "TR.JACCARD": lambda resp: float(resp.decode()),
    # TairSearch
    "TFT.CREATEINDEX": bool_ok,
    "TFT.UPDATEINDEX": bool_ok,
    "TFT.GETINDEX": lambda resp: None if resp is None else resp.decode(),
    "TFT.ADDDOC": lambda resp: resp.decode(),
    "TFT.MADDDOC": bool_ok,
    "TFT.DELDOC": lambda resp: int(resp.decode()),
    "TFT.UPDATEDOCFIELD": bool_ok,
    "TFT.INCRFLOATDOCFIELD": lambda resp: float(resp.decode()),
    "TFT.GETDOC": lambda resp: None if resp is None else resp.decode(),
    "TFT.SCANDOCID": lambda resp: ScandocidResult(
        resp[0].decode(), [i.decode() for i in resp[1]]
    ),
    "TFT.DELALL": bool_ok,
    "TFT.SEARCH": lambda resp: resp.decode(),
    "TFT.GETSUG": lambda resp: [i.decode() for i in resp],
    "TFT.GETALLSUGS": lambda resp: [i.decode() for i in resp],
    # TairDoc
    "JSON.SET": lambda resp: None if resp is None else resp == b"OK",
    "JSON.TYPE": str_if_bytes,
    # TairTs
    "EXTS.P.CREATE": bool_ok,
    "EXTS.S.CREATE": bool_ok,
    "EXTS.S.ALTER": bool_ok,
    "EXTS.S.ADD": bool_ok,
    "EXTS.S.MADD": lambda resp: [bool_ok(i) for i in resp],
    "EXTS.S.INCRBY": bool_ok,
    "EXTS.S.MINCRBY": lambda resp: [bool_ok(i) for i in resp],
    "EXTS.S.DEL": bool_ok,
    "EXTS.S.RAW_MODIFY": bool_ok,
    "EXTS.S.RAW_MMODIFY": lambda resp: [bool_ok(i) for i in resp],
    "EXTS.S.RAW_INCRBY": bool_ok,
    "EXTS.S.RAW_MINCRBY": lambda resp: [bool_ok(i) for i in resp],
    # TairCpc
    "CPC.UPDATE": bool_ok,
    "CPC.ESTIMATE": lambda resp: float(resp.decode()),
    "CPC.UPDATE2EST": lambda resp: float(resp.decode()),
    "CPC.UPDATE2JUD": lambda resp: CpcUpdate2judResult(
        float(resp[0].decode()), float(resp[1].decode())
    ),
    "CPC.ARRAY.UPDATE": bool_ok,
    "CPC.ARRAY.ESTIMATE": lambda resp: float(resp.decode()),
    "CPC.ARRAY.ESTIMATE.RANGE": lambda resp: [float(i.decode()) for i in resp],
    "CPC.ARRAY.ESTIMATE.RANGE.MERGE": lambda resp: float(resp.decode()),
    "CPC.ARRAY.UPDATE2EST": lambda resp: float(resp.decode()),
    "CPC.ARRAY.UPDATE2JUD": lambda resp: CpcUpdate2judResult(
        float(resp[0].decode()), float(resp[1].decode())
    ),
    # TairVector
    "TVS.CREATEINDEX": bool_ok,
    "TVS.GETINDEX": parse_tvs_get_index_result,
    "TVS.DELINDEX": int_or_none,
    "TVS.HSET": int_or_none,
    "TVS.DEL": int_or_none,
    "TVS.HDEL": int_or_none,
    "TVS.HGETALL": parse_tvs_get_result,
    "TVS.HMGET": parse_tvs_hmget_result,
    "TVS.KNNSEARCH": parse_tvs_search_result,
    "TVS.MKNNSEARCH": parse_tvs_msearch_result,
    "TVS.MINDEXKNNSEARCH": parse_tvs_search_result,
    "TVS.MINDEXMKNNSEARCH": parse_tvs_msearch_result,
}


def set_tair_response_callback(redis: Union[Redis, AsyncRedis]):
    for cmd, cb in TAIR_RESPONSE_CALLBACKS.items():
        redis.set_response_callback(cmd, cb)



from typing import Dict, Iterable, List, Optional, Union

from tair.exceptions import DataError
from tair.typing import CommandsProtocol, EncodableT, KeyT, ResponseT


class TairTsSkeyItem:
    def __init__(self, skey: KeyT, ts: Union[int, str], value: float) -> None:
        self.skey = skey
        self.ts = ts
        self.value = value

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, TairTsSkeyItem):
            return False
        return (
            self.skey == other.skey
            and self.ts == other.ts
            and self.value == other.value
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        return f"{{skey: {self.skey}, ts: {self.ts}, value: {self.value}}}"


class Aggregation:
    def __init__(self, aggregation_type: str, time_bucket: int) -> None:
        self.aggregation_type = aggregation_type
        self.time_bucket = time_bucket

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Aggregation):
            return False
        return (
            self.aggregation_type == other.aggregation_type
            and self.time_bucket == other.time_bucket
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        return f"{{aggregation_type: {self.aggregation_type}, time_bucket: {self.time_bucket}}}"


class TairTsCommands(CommandsProtocol):
    def exts_p_create(self, pkey: KeyT) -> ResponseT:
        return self.execute_command("EXTS.P.CREATE", pkey)

    def exts_s_create(
        self,
        pkey: KeyT,
        skey: KeyT,
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, skey]

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            for label, val in labels.items():
                pieces.append("LABELS")
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.CREATE", *pieces)

    def exts_s_alter(
        self,
        pkey: KeyT,
        skey: KeyT,
        data_et: Optional[int] = None,
    ) -> ResponseT:
        if data_et is not None:
            return self.execute_command("EXTS.S.ALTER", pkey, skey, "DATA_ET", data_et)
        return self.execute_command("EXTS.S.ALTER", pkey, skey)

    def exts_s_add(
        self,
        pkey: KeyT,
        skey: KeyT,
        ts: Union[int, str],
        value: float,
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, skey, ts, value]

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.ADD", *pieces)

    def exts_s_madd(
        self,
        pkey: KeyT,
        items: Iterable[TairTsSkeyItem],
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, len(items)]

        for item in items:
            pieces.append(item.skey)
            pieces.append(item.ts)
            pieces.append(item.value)

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.MADD", *pieces)

    def exts_s_incrby(
        self,
        pkey: KeyT,
        skey: KeyT,
        ts: Union[int, str],
        value: float,
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, skey, ts, value]

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.INCRBY", *pieces)

    def exts_s_mincrby(
        self,
        pkey: KeyT,
        items: Iterable[TairTsSkeyItem],
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, len(items)]

        for item in items:
            pieces.append(item.skey)
            pieces.append(item.ts)
            pieces.append(item.value)

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.MINCRBY", *pieces)

    def exts_s_del(self, pkey: KeyT, skey: KeyT) -> ResponseT:
        return self.execute_command("EXTS.S.DEL", pkey, skey)

    def exts_s_get(self, pkey: KeyT, skey: KeyT) -> ResponseT:
        return self.execute_command("EXTS.S.GET", pkey, skey)

    def exts_s_info(self, pkey: KeyT, skey: KeyT) -> ResponseT:
        return self.execute_command("EXTS.S.INFO", pkey, skey)

    def exts_s_queryindex(
        self,
        pkey: KeyT,
        filters: Iterable[str],
    ) -> ResponseT:
        return self.execute_command("EXTS.S.QUERYINDEX", pkey, *filters)

    def exts_s_range(
        self,
        pkey: KeyT,
        skey: KeyT,
        from_ts: int,
        to_ts: Union[int, str],
        maxcount: Optional[int] = None,
        aggregation: Optional[Aggregation] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, skey, from_ts, to_ts]

        if maxcount is not None:
            pieces.append("MAXCOUNT")
            pieces.append(maxcount)

        if aggregation is not None:
            pieces.append("AGGREGATION")
            pieces.append(aggregation.aggregation_type)
            pieces.append(aggregation.time_bucket)

        return self.execute_command("EXTS.S.RANGE", *pieces)

    def exts_s_mrange(
        self,
        pkey: KeyT,
        from_ts: int,
        to_ts: Union[int, str],
        filters: Iterable[str],
        maxcount: Optional[int] = None,
        aggregation: Optional[Aggregation] = None,
        withlabels: bool = False,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, from_ts, to_ts]

        if maxcount is not None:
            pieces.append("MAXCOUNT")
            pieces.append(maxcount)

        if aggregation is not None:
            pieces.append("AGGREGATION")
            pieces.append(aggregation.aggregation_type)
            pieces.append(aggregation.time_bucket)

        if withlabels:
            pieces.append("WITHLABELS")

        pieces.append("FILTER")
        pieces.extend(filters)

        return self.execute_command("EXTS.S.MRANGE", *pieces)

    def exts_p_range(
        self,
        pkey: KeyT,
        from_ts: int,
        to_ts: Union[int, str],
        pkey_aggregation: str = None,
        pkey_time_bucket: int = None,
        filters: Iterable[str] = None,
        maxcount: Optional[int] = None,
        aggregation: Optional[Aggregation] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, from_ts, to_ts]

        if pkey_aggregation is not None:
            pieces.append(pkey_aggregation)
            if pkey_time_bucket is not None:
                pieces.append(pkey_time_bucket)
            else:
                raise DataError(
                    "pkey_time_bucket is required when pkey_aggregation is specified"
                )
        if maxcount is not None:
            pieces.append("MAXCOUNT")
            pieces.append(maxcount)
        if aggregation is not None:
            pieces.append("AGGREGATION")
            pieces.append(aggregation.aggregation_type)
            pieces.append(aggregation.time_bucket)

        pieces.append("FILTER")
        for i in filters:
            pieces.append(i)

        return self.execute_command("EXTS.P.RANGE", *pieces)

    def exts_s_raw_modify(
        self,
        pkey: KeyT,
        skey: KeyT,
        ts: Union[int, str],
        value: float,
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, skey, ts, value]

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.RAW_INCRBY", *pieces)

    def exts_s_raw_mmodify(
        self,
        pkey: KeyT,
        items: Iterable[TairTsSkeyItem],
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, len(items)]

        for item in items:
            pieces.append(item.skey)
            pieces.append(item.ts)
            pieces.append(item.value)

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.RAW_MMODIFY", *pieces)

    def exts_s_raw_incrby(
        self,
        pkey: KeyT,
        skey: KeyT,
        ts: Union[int, str],
        value: float,
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, skey, ts, value]

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.RAW_INCRBY", *pieces)

    def exts_s_raw_mincrby(
        self,
        pkey: KeyT,
        items: Iterable[TairTsSkeyItem],
        data_et: Optional[int] = None,
        chunk_size: Optional[int] = None,
        uncompressed: bool = False,
        labels: Optional[Dict[str, str]] = None,
    ) -> ResponseT:
        pieces: List[EncodableT] = [pkey, len(items)]

        for item in items:
            pieces.append(item.skey)
            pieces.append(item.ts)
            pieces.append(item.value)

        if data_et is not None:
            pieces.append("DATA_ET")
            pieces.append(data_et)
        if chunk_size is not None:
            pieces.append("CHUNK_SIZE")
            pieces.append(chunk_size)
        if uncompressed:
            pieces.append("UNCOMPRESSED")

        if labels is not None:
            pieces.append("LABELS")
            for label, val in labels.items():
                pieces.append(label)
                pieces.append(val)

        return self.execute_command("EXTS.S.RAW_MINCRBY", *pieces)



import uuid

import pytest
from redis.exceptions import RedisClusterException

from tair import ExgetResult, Tair, TairCluster


class TestPipeline:
    def test_pipeline_is_true(self, t: Tair):
        with t.pipeline() as pipe:
            assert pipe

    def test_pipeline(self, t: Tair):
        with t.pipeline() as pipe:
            key1 = "key_" + str(uuid.uuid4())
            key2 = "key_" + str(uuid.uuid4())
            key3 = "key_" + str(uuid.uuid4())
            value1 = "value_" + str(uuid.uuid4())
            value2 = "value_" + str(uuid.uuid4())
            value3 = "value_" + str(uuid.uuid4())

            (
                pipe.exset(key1, value1)
                .exset(key2, value2)
                .exset(key3, value3)
                .exget(key1)
                .exget(key2)
                .exget(key3)
            )
            assert pipe.execute() == [
                True,
                True,
                True,
                ExgetResult(value1.encode(), 1),
                ExgetResult(value2.encode(), 1),
                ExgetResult(value3.encode(), 1),
            ]

    def test_pipeline_cluster(self, tc: TairCluster):
        with tc.pipeline() as pipe:
            key1 = "key_" + str(uuid.uuid4())
            key2 = "key_" + str(uuid.uuid4())
            key3 = "key_" + str(uuid.uuid4())
            value1 = "value_" + str(uuid.uuid4())
            value2 = "value_" + str(uuid.uuid4())
            value3 = "value_" + str(uuid.uuid4())

            (
                pipe.exset(key1, value1)
                .exset(key2, value2)
                .exset(key3, value3)
                .exget(key1)
                .exget(key2)
                .exget(key3)
            )
            assert pipe.execute() == [
                True,
                True,
                True,
                ExgetResult(value1.encode(), 1),
                ExgetResult(value2.encode(), 1),
                ExgetResult(value3.encode(), 1),
            ]

    def test_deprecated(self, tc: TairCluster):
        with pytest.raises(RedisClusterException):
            tc.pipeline(transaction=True)
        with pytest.raises(RedisClusterException):
            tc.pipeline(shard_hint=True)



import datetime
import time
import uuid

import pytest

from tair import DataError, ExcasResult, ExgetResult, ResponseError, Tair

from .conftest import NETWORK_DELAY_CALIBRATION_VALUE, get_server_time


class TestTairString:
    def test_exset_success(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.exset(key, value)
        result = t.exget(key)
        assert result.value == value.encode()
        assert result.version == 1

    def test_exset_ex(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.exset(key, value, ex=10)
        assert 0 < t.ttl(key) <= 10

        # ex should not be a float.
        with pytest.raises(DataError):
            t.exset(key, value, ex=10.0)

    def test_exset_ex_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())
        expire_at = datetime.timedelta(seconds=10)

        assert t.exset(key, value, ex=expire_at)
        assert 0 < t.ttl(key) <= 10

    def test_exset_px(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.exset(key, value, px=10000)
        assert 0 < t.pttl(key) <= 10000

        # px should not be a float.
        with pytest.raises(DataError):
            t.exset(key, value, px=10000.0)

    def test_exset_px_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())
        expire_at = datetime.timedelta(milliseconds=10000)

        assert t.exset(key, value, px=expire_at)
        assert 0 < t.pttl(key) <= 10000

    def test_exset_exat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)
        exat = int(time.mktime(expire_at.timetuple()))

        assert t.exset(key, value, exat=exat)
        assert 0 < t.ttl(key) <= 10

    def test_exset_exat_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)

        assert t.exset(key, value, exat=expire_at)
        assert 0 < t.ttl(key) <= 10

    def test_exset_pxat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)
        pxat = int(time.mktime(expire_at.timetuple())) * 1000

        assert t.exset(key, value, pxat=pxat)
        # due to network delay, pttl may be greater than 10000.
        assert 0 < t.pttl(key) <= (10000 + NETWORK_DELAY_CALIBRATION_VALUE)

    def test_exset_pxat_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)

        assert t.exset(key, value, pxat=expire_at)
        # due to network delay, pttl may be greater than 10000.
        assert 0 < t.pttl(key) <= (10000 + NETWORK_DELAY_CALIBRATION_VALUE)

    def test_exset_nx(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.exset(key, value, nx=True)
        # if the key exists and nx is True, exset will return None.
        assert t.exset(key, value, nx=True) is None

    def test_exset_xx(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value1 = "value_" + str(uuid.uuid4())
        value2 = "value_" + str(uuid.uuid4())

        # if the key does not exists and xx is True, exset will return None.
        assert t.exset(key, value1, xx=True) is None
        assert t.exset(key, value1)
        assert t.exset(key, value2, xx=True)

    def test_exset_ver(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value1 = "value_" + str(uuid.uuid4())
        value2 = "value_" + str(uuid.uuid4())

        assert t.exset(key, value1)
        assert t.exset(key, value2, ver=1)
        result = t.exget(key)
        assert result.value == value2.encode()
        assert result.version == 2

    def test_exset_abs(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.exset(key, value, abs=10)
        result = t.exget(key)
        assert result.value == value.encode()
        assert result.version == 10

    def test_exsetver(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.exset(key1, value)
        assert t.exsetver(key1, 10) == 1
        result = t.exget(key1)
        assert result.value == value.encode()
        assert result.version == 10

        # if the key does not exists, exsetver will return 0.
        assert t.exsetver(key2, 10) == 0

    def test_exincrby_success(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 10)
        assert t.exincrby(key, 20) == 30
        result = t.exget(key)
        assert result.value == b"30"
        assert result.version == 2

    def test_exincrby_ex(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, ex=10) == 30
        assert 0 < t.ttl(key) <= 10

        # ex should not be a float.
        with pytest.raises(DataError):
            t.exincrby(key, 20, ex=10.0)

    def test_exincrby_ex_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = datetime.timedelta(seconds=10)

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, ex=expire_at) == 30
        assert 0 < t.ttl(key) <= 10

    def test_exincrby_px(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, px=10000) == 30
        assert 0 < t.pttl(key) <= 10000

        # px should not be a float.
        with pytest.raises(DataError):
            t.exincrby(key, 20, px=10000.0)

    def test_exincrby_px_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = datetime.timedelta(milliseconds=10000)

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, px=expire_at) == 30
        assert 0 < t.pttl(key) <= 10000

    def test_exincrby_exat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)
        exat = int(time.mktime(expire_at.timetuple()))

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, exat=exat) == 30
        assert 0 < t.ttl(key) <= 10

    def test_exincrby_exat_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, exat=expire_at) == 30
        assert 0 < t.ttl(key) <= 10

    def test_exincrby_pxat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)
        pxat = int(time.mktime(expire_at.timetuple())) * 1000

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, pxat=pxat) == 30
        # due to network delay, pttl may be greater than 10000.
        assert 0 < t.pttl(key) <= (10000 + NETWORK_DELAY_CALIBRATION_VALUE)

    def test_exincrby_pxat_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, pxat=expire_at) == 30
        # due to network delay, pttl may be greater than 10000.
        assert 0 < t.pttl(key) <= (10000 + NETWORK_DELAY_CALIBRATION_VALUE)

    def test_exincrby_nx(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.exincrby(key1, 20, nx=True) == 20
        result = t.exget(key1)
        assert result.value == b"20"
        assert result.version == 1

        assert t.exset(key2, 10)
        assert t.exincrby(key2, 20, nx=True) is None
        result = t.exget(key2)
        assert result.value == b"10"
        assert result.version == 1

    def test_exincrby_xx(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.exincrby(key1, 20, xx=True) is None
        assert not t.exists(key1)

        assert t.exset(key2, 10)
        assert t.exincrby(key2, 20, xx=True) == 30
        result = t.exget(key2)
        assert result.value == b"30"
        assert result.version == 2

    def test_exincrby_ver(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, ver=1) == 30
        result = t.exget(key)
        assert result.value == b"30"
        assert result.version == 2

        assert t.exset(key, 10)
        with pytest.raises(ResponseError):
            t.exincrby(key, 20, ver=10)

    def test_exincrby_abs(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 10)
        assert t.exincrby(key, 20, abs=100) == 30
        result = t.exget(key)
        assert result.value == b"30"
        assert result.version == 100

    def test_exincrby_overflow(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 10)
        with pytest.raises(ResponseError):
            t.exincrby(key, 20, maxval=10)
        with pytest.raises(ResponseError):
            t.exincrby(key, 20, minval=100)

    def test_exincrbyfloat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2) == pytest.approx(3.3)
        result = t.exget(key)
        assert float(result.value) == pytest.approx(3.3)
        assert result.version == 2

    def test_exincrbyfloat_ex(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, ex=10) == pytest.approx(3.3)
        assert 0 < t.ttl(key) <= 10

        # ex should not be a float.
        with pytest.raises(DataError):
            t.exincrbyfloat(key, 20, ex=10.0)

    def test_exincrbyfloat_ex_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = datetime.timedelta(seconds=10)

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, ex=expire_at) == pytest.approx(3.3)
        assert 0 < t.ttl(key) <= 10

    def test_exincrbyfloat_px(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, px=10000) == pytest.approx(3.3)
        assert 0 < t.pttl(key) <= 10000

        # px should not be a float.
        with pytest.raises(DataError):
            t.exincrbyfloat(key, 20, px=10000.0)

    def test_exincrbyfloat_px_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = datetime.timedelta(milliseconds=10000)

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, px=expire_at) == pytest.approx(3.3)
        assert 0 < t.pttl(key) <= 10000

    def test_exincrbyfloat_exat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)
        exat = int(time.mktime(expire_at.timetuple()))

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, exat=exat) == pytest.approx(3.3)
        assert 0 < t.ttl(key) <= 10

    def test_exincrbyfloat_exat_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, exat=expire_at) == pytest.approx(3.3)
        assert 0 < t.ttl(key) <= 10

    def test_exincrbyfloat_pxat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)
        pxat = int(time.mktime(expire_at.timetuple())) * 1000

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, pxat=pxat) == pytest.approx(3.3)
        # due to network delay, pttl may be greater than 10000.
        assert 0 < t.pttl(key) <= (10000 + NETWORK_DELAY_CALIBRATION_VALUE)

    def test_exincrbyfloat_pxat_timedelta(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        expire_at = get_server_time(t) + datetime.timedelta(seconds=10)

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, pxat=expire_at) == pytest.approx(3.3)
        # due to network delay, pttl may be greater than 10000.
        assert 0 < t.pttl(key) <= (10000 + NETWORK_DELAY_CALIBRATION_VALUE)

    def test_exincrbyfloat_nx(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.exincrbyfloat(key1, 1.1, nx=True) == 1.1
        result = t.exget(key1)
        assert float(result.value) == pytest.approx(1.1)
        assert result.version == 1

        assert t.exset(key2, 1.1)
        assert t.exincrbyfloat(key2, 2.2, nx=True) is None
        result = t.exget(key2)
        assert float(result.value) == pytest.approx(1.1)
        assert result.version == 1

    def test_exincrbyfloat_xx(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.exincrbyfloat(key1, 1.1, xx=True) is None
        assert not t.exists(key1)

        assert t.exset(key2, 1.1)
        assert t.exincrbyfloat(key2, 2.2, xx=True) == pytest.approx(3.3)
        result = t.exget(key2)
        assert float(result.value) == pytest.approx(3.3)
        assert result.version == 2

    def test_exincrbyfloat_ver(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, ver=1) == pytest.approx(3.3)
        result = t.exget(key)
        assert float(result.value) == pytest.approx(3.3)
        assert result.version == 2

        assert t.exset(key, 1.1)
        with pytest.raises(ResponseError):
            t.exincrbyfloat(key, 2.2, ver=10)

    def test_exincrbyfloat_abs(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 1.1)
        assert t.exincrbyfloat(key, 2.2, abs=100) == pytest.approx(3.3)
        result = t.exget(key)
        assert float(result.value) == pytest.approx(3.3)
        assert result.version == 100

    def test_exincrbyfloat_overflow(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.exset(key, 9.1)
        with pytest.raises(ResponseError):
            t.exincrbyfloat(key, 1.1, maxval=10)
        with pytest.raises(ResponseError):
            t.exincrbyfloat(key, 0.1, minval=100)

    def test_excas_success(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value1 = "value_" + str(uuid.uuid4())
        value2 = "value_" + str(uuid.uuid4())

        assert t.exset(key, value1)
        excas_result = t.excas(key, value2, 1)
        assert excas_result.msg == "OK"
        assert excas_result.value == b""
        assert excas_result.version == 2
        exget_result = t.exget(key)
        assert exget_result.value == value2.encode()
        assert exget_result.version == 2

    def test_excas_failed(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value1 = "value_" + str(uuid.uuid4())
        value2 = "value_" + str(uuid.uuid4())

        assert t.exset(key, value1)
        result = t.excas(key, value2, 100)
        assert result.msg == "CAS_FAILED"
        assert result.value == value1.encode()
        assert result.version == 1

    def test_excas_not_exists(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        # if the key does not exist, return -1.
        assert t.excas(key, value, 1) == -1

    def test_excad(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.exset(key, value)
        assert t.excad(key, 1) == 1
        assert not t.exists(key)

    def test_excad_not_exists(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        # if the key does not exist, return -1.
        assert t.excad(key, 1) == -1

    def test_cas_success(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.set(key, value) is True
        assert t.cas(key, value, "newval") == 1

    def test_cas_fail(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.set(key, value) is True
        assert t.cas(key, "oldkey", "newval") == 0

    def test_cad_success(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.setnx(key, value) is True
        assert t.cad(key, value) == 1

    def test_cad_fail(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        value = "value_" + str(uuid.uuid4())

        assert t.setnx(key, value) is True
        assert t.cad(key, "newval") == 0

    def test_exget_result_eq(self):
        value = "value_" + str(uuid.uuid4())
        assert ExgetResult(value.encode(), 1) == ExgetResult(value.encode(), 1)
        assert not ExgetResult(value.encode(), 1) == ExgetResult(value.encode(), 2)
        assert not ExgetResult(value.encode(), 1) == 1

    def test_exget_result_ne(self):
        value = "value_" + str(uuid.uuid4())
        assert not ExgetResult(value.encode(), 1) != ExgetResult(value.encode(), 1)
        assert ExgetResult(value.encode(), 1) != ExgetResult(value.encode(), 2)
        assert ExgetResult(value.encode(), 1) != 1

    def test_exget_result_repr(self):
        value = "value_" + str(uuid.uuid4())
        assert (
            str(ExgetResult(value.encode(), 100))
            == f"{{value: {value.encode()}, version: 100}}"
        )

    def test_excas_result_eq(self):
        value = "value_" + str(uuid.uuid4())
        assert ExcasResult("OK", value.encode(), 1) == ExcasResult(
            "OK", value.encode(), 1
        )
        assert not ExcasResult("OK", value.encode(), 1) == ExcasResult(
            "OK", value.encode(), 2
        )
        assert not ExcasResult("OK", value.encode(), 1) == 1

    def test_excas_result_ne(self):
        value = "value_" + str(uuid.uuid4())
        assert not ExcasResult("OK", value.encode(), 1) != ExcasResult(
            "OK", value.encode(), 1
        )
        assert ExcasResult("OK", value.encode(), 1) != ExcasResult(
            "OK", value.encode(), 2
        )
        assert ExcasResult("OK", value.encode(), 1) != 1

    def test_excas_result_repr(self):
        value = "value_" + str(uuid.uuid4())
        assert (
            str(ExcasResult("OK", value.encode(), 100))
            == f"{{msg: OK, value: {value.encode()}, version: 100}}"
        )



import uuid

import pytest

from tair import Tair, TrScanResult


class TestTairRoaring:
    def test_tr_setbit(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbit(key, 3, 1) == 0
        assert t.tr_getbit(key, 3) == 1
        assert t.tr_setbit(key, 3, 0) == 1
        assert t.tr_getbit(key, 3) == 0

    def test_tr_setbits(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        offsets1 = [3, 4, 5]
        offsets2 = [8, 9]

        assert t.tr_setbits(key, offsets1) == 3
        assert t.tr_setbits(key, offsets2) == 5

    def test_tr_clearbits(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        offsets1 = [3, 4, 5]
        offsets2 = [4, 5, 6]

        assert t.tr_setbits(key, offsets1) == 3
        assert t.tr_clearbits(key, offsets2) == 2

    def test_tr_clearbits_not_exists(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        offsets = [3, 4, 5]

        assert t.tr_clearbits(key, offsets) == 0

    def test_tr_setrange(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setrange(key, 1, 3) == 3
        assert t.tr_getbits(key, [0, 1, 2, 3]) == [0, 1, 1, 1]

    def test_tr_appendbitarray(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        offsets = [0, 1, 2, 3, 4, 5]

        assert t.tr_setbits(key, offsets) == 6
        assert t.tr_appendbitarray(key, 1, "1101") == 5
        assert t.tr_getbits(key, offsets) == [1, 1, 1, 1, 0, 1]

    def test_tr_fliprange(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [0, 2, 3, 5]) == 4
        assert t.tr_fliprange(key, 0, 5) == 2
        assert t.tr_getbits(key, [0, 1, 2, 3, 4, 5]) == [0, 1, 0, 0, 1, 0]

    def test_tr_appendintarray(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        offsets = [9, 10]

        assert t.tr_setbit(key, 0, 1) == 0
        assert t.tr_appendintarray(key, offsets)
        assert t.tr_getbit(key, 0) == 1
        assert t.tr_getbits(key, offsets) == [1, 1]

    def test_tr_setintarray(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        offsets = [2, 4, 5, 6]

        assert t.tr_setbit(key, 0, 1) == 0
        assert t.tr_setintarray(key, offsets)
        assert t.tr_getbit(key, 0) == 0
        assert t.tr_getbits(key, offsets) == [1, 1, 1, 1]

    def test_tr_setbitarray(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        offsets = [i for i in range(8)]

        assert t.tr_setintarray(key, offsets)
        assert t.tr_getbits(key, offsets) == [1] * 8
        assert t.tr_setbitarray(key, "10101001")
        assert t.tr_getbits(key, offsets) == [1, 0, 1, 0, 1, 0, 0, 1]

    def test_tr_bitop_and(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())
        key3 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [2, 3]) == 2
        assert t.tr_bitop(key3, "AND", (key1, key2)) == 2
        assert t.tr_getbits(key3, [i for i in range(6)]) == [0, 0, 1, 1, 0, 0]

    def test_tr_bitop_or(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())
        key3 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [1, 4]) == 2
        assert t.tr_bitop(key3, "OR", (key1, key2)) == 6
        assert t.tr_getbits(key3, [i for i in range(6)]) == [1, 1, 1, 1, 1, 1]

    def test_tr_bitop_xor(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())
        key3 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [1, 4]) == 2
        assert t.tr_bitop(key3, "XOR", (key1, key2)) == 6
        assert t.tr_getbits(key3, [i for i in range(6)]) == [1, 1, 1, 1, 1, 1]

    def test_tr_bitop_not(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_bitop(key2, "NOT", (key1,)) == 2
        assert t.tr_getbits(key2, [i for i in range(6)]) == [0, 1, 0, 0, 1, 0]

    def test_tr_bitop_diff(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())
        key3 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [1, 4]) == 2
        assert t.tr_bitop(key3, "DIFF", (key1, key2)) == 4
        assert t.tr_getbits(key3, [i for i in range(6)]) == [1, 0, 1, 1, 0, 1]

    def test_tr_bitopcard_and(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [2, 3]) == 2
        assert t.tr_bitopcard("AND", (key1, key2)) == 2

    def test_tr_bitopcard_or(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [1, 4]) == 2
        assert t.tr_bitopcard("OR", (key1, key2)) == 6

    def test_tr_bitopcard_xor(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [1, 4]) == 2
        assert t.tr_bitopcard("XOR", (key1, key2)) == 6

    def test_tr_bitopcard_not(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [0, 2, 3, 5]) == 4
        assert t.tr_bitopcard("NOT", (key,)) == 2

    def test_tr_bitopcard_diff(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [0, 2, 3, 5]) == 4
        assert t.tr_setbits(key2, [1, 4]) == 2
        assert t.tr_bitopcard("DIFF", (key1, key2)) == 4

    def test_tr_optimize(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [0, 2, 3, 5]) == 4
        assert t.tr_optimize(key)

    def test_tr_bitcount(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [1, 2, 3, 10]) == 4
        assert t.tr_bitcount(key) == 4
        assert t.tr_bitcount(key, start=2, end=3) == 2

    def test_tr_bitpos(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [1, 2, 3, 10]) == 4
        assert t.tr_bitpos(key, 1, 2) == 2

    def test_tr_scan(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [1, 2, 3, 10]) == 4
        assert t.tr_scan(key, 0, 2) == TrScanResult(3, [1, 2])

    def test_tr_range(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [0, 2, 3, 5]) == 4
        assert t.tr_range(key, 0, 5) == [0, 2, 3, 5]

    def test_tr_rangebitarray(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [0, 2, 3, 5]) == 4
        assert t.tr_rangebitarray(key, 0, 5) == "101101"

    def test_tr_min(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_min(key) == -1
        assert t.tr_setbit(key, 0, 0) == 0
        assert t.tr_min(key) == -1
        assert t.tr_setbits(key, [1, 2, 3, 10]) == 4
        assert t.tr_min(key) == 1

    def test_tr_max(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_max(key) == -1
        assert t.tr_setbit(key, 0, 0) == 0
        assert t.tr_max(key) == -1
        assert t.tr_setbits(key, [1, 2, 3, 10]) == 4
        assert t.tr_max(key) == 10

    def test_tr_stat(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [1, 2, 3, 5, 8]) == 5
        assert type(t.tr_stat(key)) == bytes
        assert type(t.tr_stat(key, json=True)) == bytes

    def test_tr_jaccard(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())
        key3 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [1, 2, 3, 5, 8]) == 5
        assert t.tr_setbits(key2, [1, 2, 3, 5, 8]) == 5
        assert t.tr_setbits(key3, [1, 3, 5]) == 3
        assert t.tr_jaccard(key1, key2) == pytest.approx(1.0)
        assert t.tr_jaccard(key2, key3) == pytest.approx(0.6)

    def test_tr_contains(self, t: Tair):
        key1 = "key_" + str(uuid.uuid4())
        key2 = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key1, [1, 2, 3, 10]) == 4
        assert t.tr_setbits(key2, [1, 2]) == 2
        assert t.tr_contains(key1, key2) == 0
        assert t.tr_contains(key2, key1) == 1

    def test_tr_rank(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.tr_setbits(key, [1, 2, 3, 10]) == 4
        assert t.tr_rank(key, 10) == 4

    def test_tr_scan_result_eq(self):
        assert TrScanResult(3, [1, 2]) == TrScanResult(3, [1, 2])
        assert not TrScanResult(3, [1, 2]) == TrScanResult(3, [1, 3])
        assert not TrScanResult(3, [1, 2]) == 1

    def test_tr_scan_result_ne(self):
        assert not TrScanResult(3, [1, 2]) != TrScanResult(3, [1, 2])
        assert TrScanResult(3, [1, 2]) != TrScanResult(3, [1, 3])
        assert TrScanResult(3, [1, 2]) != 1

    def test_tr_scan_result_repr(self):
        assert str(TrScanResult(3, [1, 2])) == f"{{start_offset: 3, offsets: [1, 2]}}"



# /user/bin/env python3
import os
import string
import sys
import unittest
import uuid
from random import choice, randint, random

import redis

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from tair.tairvector import (
    Constants,
    DataType,
    DistanceMetric,
    TairVectorCommands,
    TairVectorIndex,
)

from .conftest import get_tair_client

client = get_tair_client()

dim = 16
num_vectors = 100
test_vectors = [[random() for _ in range(dim)] for _ in range(num_vectors)]
test2_vectors = [[random() for _ in range(dim)] for _ in range(num_vectors)]
num_attrs = 3
attr_keys = ["key-%d" % i for i in range(num_attrs)]
attr_values = [
    "".join(choice(string.ascii_uppercase + string.digits) for _ in range(4))
    for _ in range(num_vectors * num_attrs)
]
test_attributes = [
    dict(zip(attr_keys, attr_values[i : i + 3]))
    for i in range(0, num_vectors * num_attrs, num_attrs)
]

num_queries = 10
queries = [[random() for _ in range(dim)] for _ in range(num_queries)]


class IndexCommandsTest(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super().__init__(methodName=methodName)
        self.index_params = {"M": 32, "ef_construct": 200}

    # the following test functions will execute in alphabetical order
    def test_0_delete_all_indices(self):
        # get all indices
        indices = []
        result = client.tvs_scan_index()
        for index in result.iter():
            indices.append(index)

        print("deleting indices:", indices)
        # deleting all indices
        for index in indices:
            try:
                ret = client.tvs_del_index(index)
                self.assertEqual(ret, 1)
            except:
                self.logger.error("delete index [%s] failed" % index)

        # scan indices again
        indices = []
        result = client.tvs_scan_index()
        for index in result.iter():
            indices.append(index)
        self.assertEqual(indices, [])

    def test_1_delete_noexist_index(self):
        print(
            "deleting no-exist index",
        )
        ret = client.tvs_del_index("test")
        self.assertFalse(ret, 0)

    def test_2_get_nonexist_index(self):
        self.assertIsNone(client.tvs_get_index("test"))

    def test_3_create_index(self):
        ret = client.tvs_create_index("test", dim, **self.index_params)
        self.assertEqual(ret, True)

    def test_4_create_duplicate_index(self):
        with self.assertRaises(redis.exceptions.ResponseError):
            ret = client.tvs_create_index("test", dim, **self.index_params)

    def test_5_get_index(self):
        index = client.tvs_get_index("test")
        for k, v in self.index_params.items():
            self.assertTrue(k in index)
            self.assertEqual(index[k], str(v))

    def test_6_scan_index(self):
        indices = []
        result = client.tvs_scan_index()
        for index in result.iter():
            indices.append(index.decode())
        self.assertListEqual(indices, ["test"])

    def test_7_scan_index_with_pattern(self):
        indices = []
        result = client.tvs_scan_index(pattern="aaa")
        for index in result.iter():
            indices.append(index)
        self.assertEqual(indices, [])

    def test_8_delete_test_index(self):
        self.assertEqual(client.tvs_del_index("test"), 1)


def floatEqual(v1: float, v2: float, epsilon=1e-6) -> bool:
    delta = v1 - v2
    return delta >= -epsilon and delta <= epsilon


def vectorEqual(v1, v2) -> bool:
    if len(v1) != len(v2):
        return False
    for i in range(len(v1)):
        if not floatEqual(v1[i], v2[i]):
            return False
    return True


class DataCommandsTest(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super().__init__(methodName=methodName)
        self.index_params = {
            "M": 32,
            "ef_construct": 200,
        }

    def test_0_init(self):
        # delete test index
        try:
            client.tvs_del_index("test")
        except:
            pass

    def test_1_hset_with_no_vector(self):
        self.assertTrue(client.tvs_create_index("test", dim, **self.index_params))
        for i, attrs in enumerate(test_attributes):
            ret = client.tvs_hset("test", str(i), vector=None, **attrs)
            self.assertEqual(ret, len(attrs))
        # get and check
        for i, attrs in enumerate(test_attributes):
            obj = client.tvs_hgetall("test", str(i))
            self.assertDictEqual(obj, attrs)
        self.assertEqual(client.tvs_del_index("test"), 1)

    def test_2_hset_with_no_attributes(self):
        self.assertTrue(client.tvs_create_index("test", dim, **self.index_params))
        for i, v in enumerate(test_vectors):
            ret = client.tvs_hset("test", str(i), vector=v)
            self.assertTrue(ret)
        # get and check
        for i, v in enumerate(test_vectors):
            obj = client.tvs_hgetall("test", str(i))
            self.assertTrue(Constants.VECTOR_KEY in obj)
            self.assertTrue(vectorEqual(v, obj[Constants.VECTOR_KEY]))

        self.assertEqual(client.tvs_del_index("test"), 1)

    def test_3_hset(self):
        self.assertTrue(client.tvs_create_index("test", dim, **self.index_params))
        for i, v in enumerate(test_vectors):
            ret = client.tvs_hset("test", str(i), vector=v, **test_attributes[i])
            self.assertTrue(ret)
        # get and check
        for i, v in enumerate(test_vectors):
            obj = client.tvs_hgetall("test", str(i))
            self.assertTrue(Constants.VECTOR_KEY in obj)
            self.assertTrue(vectorEqual(v, obj[Constants.VECTOR_KEY]))
            del obj[Constants.VECTOR_KEY]
            self.assertDictEqual(obj, test_attributes[i])

    def test_hmget(self):
        self.assertTrue(client.tvs_create_index("test", dim, **self.index_params))
        vector = [randint(1, 100) for _ in range(dim)]
        key = "key_" + str(uuid.uuid4())
        value1 = "value_" + str(uuid.uuid4())
        value2 = "value_" + str(uuid.uuid4())
        ret = client.tvs_hset("test", key, vector=vector, field1=value1, field2=value2)
        self.assertTrue(ret)
        obj = client.tvs_hmget(
            "test", key, Constants.VECTOR_KEY, "field1", "field2", "field3"
        )
        self.assertEqual(len(obj[0].split(b",")), len(vector))
        self.assertEqual(obj[1], bytes(value1, encoding="ascii"))
        self.assertEqual(obj[2], bytes(value2, encoding="ascii"))

    def test_4_scan(self):
        result = client.tvs_scan("test")
        scanned_keys = []
        for k in result.iter():
            scanned_keys.append(k.decode())
        expected_keys = [str(i) for i in range(len(test_vectors))]
        self.assertSetEqual(set(scanned_keys), set(expected_keys))

    def test_5_scan_with_pattern(self):
        result = client.tvs_scan("test", pattern="aaa")
        scanned_keys = []
        for k in result.iter():
            scanned_keys.append(k)
        self.assertEqual(scanned_keys, [])

        result = client.tvs_scan("test", pattern="0")
        scanned_keys = []
        for k in result.iter():
            scanned_keys.append(k)
        self.assertEqual(scanned_keys, [b"0"])

    def test_6_hdel(self):
        for i, attr in enumerate(test_attributes):
            keys = attr.keys()
            self.assertEqual(client.tvs_hdel("test", str(i), *keys), len(keys))

    def test_7_delete(self):
        # delete inserted entries
        for i in range(len(test_vectors)):
            self.assertEqual(client.tvs_del("test", str(i)), 1)

    def test_9_delete(self):
        self.assertEqual(client.tvs_del_index("test"), 1)


class SearchCommandsTest(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super().__init__(methodName=methodName)
        self.top_k = 10
        self.index_params = {
            "M": 32,
            "ef_construct": 200,
        }

    def test_0_init(self):
        # delete test index
        try:
            client.tvs_del_index("test")
            client.tvs_del_index("test2")
        except:
            pass

        ret = client.tvs_create_index("test", dim, **self.index_params)
        ret = client.tvs_create_index("test2", dim, **self.index_params)
        if not ret:
            raise RuntimeError("create test/test2 index failed")

    def test_1_insert_vectors(self):
        for i, v in enumerate(test_vectors):
            ret = client.tvs_hset("test", str(i).zfill(6), vector=v)
            self.assertTrue(ret)
        for i, v in enumerate(test2_vectors):
            ret = client.tvs_hset("test2", str(i).zfill(6), vector=v)
            self.assertTrue(ret)

    def test_2_knn_search(self):
        for q in queries:
            result = client.tvs_knnsearch("test", self.top_k, vector=q)
            self.assertEqual(self.top_k, len(result))
            d = 0.0
            for k, v in result:
                self.assertGreaterEqual(v, d)
                d = v

    def test_3_knn_search_with_params(self):
        for ef in range(self.top_k, 100, 10):
            for q in queries:
                result = client.tvs_knnsearch(
                    "test", self.top_k, vector=q, ef_search=ef
                )
                self.assertEqual(self.top_k, len(result))
                d = 0.0
                for k, v in result:
                    self.assertGreaterEqual(v, d)
                    d = v

    def test_4_knn_msearch(self):
        batch = queries[:2]
        result = client.tvs_mknnsearch("test", self.top_k, batch)
        self.assertEqual(len(result), len(batch))
        for r in result:
            d = 0.0
            for _, v in r:
                self.assertGreaterEqual(v, d)
                d = v

    # todo
    def test_5_search_with_filters(self):
        pass

    def test_6_msearch_with_filters(self):
        pass

    def test_7_mindexknnsearch(self):
        indexs = ["test", "test2"]
        for q in queries:
            result = client.tvs_mindexknnsearch(indexs, self.top_k, vector=q)
            self.assertEqual(self.top_k, len(result))
            d = 0.0
            for k, v in result:
                self.assertGreaterEqual(v, d)
                d = v

    def test_8_mindexmknnsearch(self):
        indexs = ["test", "test2"]
        batch = queries[:2]
        result = client.tvs_mindexmknnsearch(indexs, self.top_k, batch)
        self.assertEqual(len(result), len(batch))
        for r in result:
            d = 0.0
            for _, v in r:
                self.assertGreaterEqual(v, d)
                d = v

    def test_9_delete_index(self):
        client.tvs_del_index("test")
        client.tvs_del_index("test2")


class IndexApiTest(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super().__init__(methodName=methodName)
        self.index_params = {
            "M": 32,
            "ef_construct": 200,
        }

    def test_0_init(self):
        client.tvs_del_index("test")
        # try getting a non-existing index
        with self.assertRaises(ValueError):
            index = TairVectorIndex(client, "test")

    def test_1_create_index(self):
        # create a new index
        index = TairVectorIndex(client, "test", dim=dim, **self.index_params)
        self.assertIsNotNone(index)
        print(index)

        # get an existing index
        index2 = TairVectorIndex(client, "test")
        self.assertEqual(str(index), str(index2))

    def test_2_create_duplicate_index(self):
        with self.assertRaises(redis.exceptions.ResponseError):
            index = TairVectorIndex(client, "test", dim=dim, **self.index_params)

    def test_3_index_api(self):
        index = client.tvs_index("test")

        for i, v in enumerate(test_vectors):
            ret = index.tvs_hset(str(i), vector=v, **test_attributes[i])
            self.assertTrue(ret)
        # get and check
        for i, v in enumerate(test_vectors):
            obj = index.tvs_hgetall(str(i))
            self.assertTrue(Constants.VECTOR_KEY in obj)
            self.assertTrue(vectorEqual(v, obj[Constants.VECTOR_KEY]))
            del obj[Constants.VECTOR_KEY]
            self.assertDictEqual(obj, test_attributes[i])

        result = index.tvs_scan()
        scanned_keys = []
        for k in result.iter():
            scanned_keys.append(k.decode())
        expected_keys = [str(i) for i in range(len(test_vectors))]
        self.assertSetEqual(set(scanned_keys), set(expected_keys))

        for i, attr in enumerate(test_attributes):
            keys = attr.keys()
            self.assertEqual(index.tvs_hdel(str(i), *keys), len(keys))

        # delete inserted entries
        for i in range(len(test_vectors)):
            self.assertEqual(index.tvs_del(str(i)), 1)


dim_bin_vector = 16
num_bin_vectors = 1000
test_bin_vectors = [
    [randint(0, 1) for _ in range(dim_bin_vector)] for _ in range(num_bin_vectors)
]


def jaccard(x, y):
    intersect = 0
    union = 0
    for i in range(len(x)):
        if x[i] == 1 or y[i] == 1:
            union += 1
        if x[i] == 1 and y[i] == 1:
            intersect += 1
    return 1 - intersect / union


class BinaryIndexTest(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super().__init__(methodName=methodName)
        self.index_params = {
            "M": 32,
            "ef_construct": 200,
        }

    def test_0_create(self):
        ret = client.tvs_create_index(
            "test_bin",
            dim_bin_vector,
            distance_type=DistanceMetric.Jaccard,
            data_type=DataType.Binary,
            **self.index_params
        )
        self.assertTrue(ret)

        # try create index with invalid parameters
        with self.assertRaises(redis.exceptions.ResponseError):
            ret = client.tvs_create_index(
                "test_bin2",
                dim_bin_vector,
                distance_type=DistanceMetric.L2,
                data_type=DataType.Binary,
                **self.index_params
            )

    def test_1_hset_hget(self):
        for i, v in enumerate(test_bin_vectors):
            ret = client.tvs_hset("test_bin", str(i), v, True)
            self.assertEqual(ret, 1)
        for i, v in enumerate(test_bin_vectors):
            ret = client.tvs_hgetall("test_bin", str(i))
            self.assertEqual(v, list(ret[Constants.VECTOR_KEY]))

    def test_2_knnsearch(self):
        q = [randint(0, 1) for _ in range(dim_bin_vector)]
        ret = client.tvs_knnsearch("test_bin", 10, q, True)
        last_dist = 0.0

        for k, s in ret:
            self.assertGreaterEqual(s, last_dist)
            last_dist = s
            self.assertAlmostEqual(s, jaccard(q, test_bin_vectors[int(k)]))

    def test_3_index_api(self):
        ret = client.tvs_del_index("test_bin")
        self.assertEqual(ret, 1)
        with self.assertRaises(ValueError):
            index = TairVectorIndex(client, "test_bin")

        index = TairVectorIndex(
            client,
            "test_bin",
            dim=dim_bin_vector,
            distance_type=DistanceMetric.Jaccard,
            data_type=DataType.Binary,
            **self.index_params
        )
        self.assertIsNotNone(index)

        index2 = TairVectorIndex(client, "test_bin")
        self.assertEqual(str(index), str(index2))

        for i, v in enumerate(test_bin_vectors):
            ret = index.tvs_hset(str(i), v)
            self.assertEqual(ret, 1)

        for i, v in enumerate(test_bin_vectors):
            ret = index.tvs_hgetall(str(i))
            self.assertEqual(v, list(ret[Constants.VECTOR_KEY]))

        q = [randint(0, 1) for _ in range(dim_bin_vector)]
        ret = index.tvs_knnsearch(10, q)
        last_dist = 0.0

        for k, s in ret:
            self.assertGreaterEqual(s, last_dist)
            last_dist = s
            self.assertAlmostEqual(s, jaccard(q, test_bin_vectors[int(k)]))

    def test_9_cleanup(self):
        ret = client.tvs_del_index("test_bin")
        self.assertEqual(ret, 1)


if __name__ == "__main__":
    unittest.main()
    client.close()



from typing import Dict, List, Optional

from tair.typing import CommandsProtocol, EncodableT, FieldT, KeyT, ResponseT


class TairGisSearchRadius:
    def __init__(
        self,
        longitude: float,
        latitude: float,
        distance: float,
        unit: str,
    ) -> None:
        self.longitude = longitude
        self.latitude = latitude
        self.distance = distance
        self.unit = unit

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, TairGisSearchRadius):
            return False
        return (
            self.longitude == other.longitude
            and self.latitude == other.latitude
            and self.distance == other.distance
            and self.unit == other.unit
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        return (
            "{"
            + f"longitude: {self.longitude}, "
            + f"latitude: {self.latitude}, "
            + f"distance: {self.distance}, "
            + f"unit: {self.unit}"
            + "}"
        )


class TairGisSearchMember:
    def __init__(self, field: FieldT, distance: float, unit) -> None:
        self.field = field
        self.distance = distance
        self.unit = unit

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, TairGisSearchMember):
            return False
        return (
            self.field == other.field
            and self.distance == other.distance
            and self.unit == other.unit
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        return (
            "{"
            + f"field: {self.field}, "
            + f"distance: {self.distance}, "
            + f"unit: {self.unit}"
            + "}"
        )


class TairGisCommands(CommandsProtocol):
    def gis_add(self, area: KeyT, mapping: Dict[KeyT, str]) -> ResponseT:
        pieces: List[EncodableT] = [area]

        for name, wkt in mapping.items():
            pieces.append(name)
            pieces.append(wkt)

        return self.execute_command("GIS.ADD", *pieces)

    def gis_get(self, area: KeyT, polygon_name: EncodableT) -> ResponseT:
        return self.execute_command("GIS.GET", area, polygon_name)

    def gis_getall(self, area: KeyT, withoutwkts: bool = False) -> ResponseT:
        if withoutwkts:
            return self.execute_command("GIS.GETALL", area, "WITHOUTWKT")
        return self.execute_command("GIS.GETALL", area)

    def gis_contains(
        self,
        area: KeyT,
        polygon_wkt: EncodableT,
        withoutwkts: bool = False,
    ) -> ResponseT:
        if withoutwkts:
            return self.execute_command(
                "GIS.CONTAINS",
                area,
                polygon_wkt,
                "WITHOUTWKT",
            )
        return self.execute_command("GIS.CONTAINS", area, polygon_wkt)

    def gis_within(
        self,
        area: KeyT,
        polygon_wkt: EncodableT,
        withoutwkts: bool = False,
    ) -> ResponseT:
        if withoutwkts:
            return self.execute_command(
                "GIS.WITHIN",
                area,
                polygon_wkt,
                "WITHOUTWKT",
            )
        return self.execute_command("GIS.WITHIN", area, polygon_wkt)

    def gis_intersects(
        self,
        area: KeyT,
        polygon_wkt: EncodableT,
        withoutwkts: bool = False,
    ) -> ResponseT:
        if withoutwkts:
            return self.execute_command(
                "GIS.INTERSECTS",
                area,
                polygon_wkt,
                "WITHOUTWKT",
            )
        return self.execute_command("GIS.INTERSECTS", area, polygon_wkt)

    def gis_search(
        self,
        area: KeyT,
        radius: Optional[TairGisSearchRadius] = None,
        member: Optional[TairGisSearchMember] = None,
        geom: Optional[EncodableT] = None,
        count: Optional[int] = None,
        asc: bool = True,
        desc: bool = False,
        withdist: bool = False,
        withoutwkts: bool = False,
    ) -> ResponseT:
        pieces: List[EncodableT] = [area]

        if radius:
            pieces.append("RADIUS")
            pieces.append(radius.longitude)
            pieces.append(radius.latitude)
            pieces.append(radius.distance)
            pieces.append(radius.unit)
        if member:
            pieces.append("MEMBER")
            pieces.append(member.field)
            pieces.append(member.distance)
            pieces.append(member.unit)
        if geom:
            pieces.append("GEOM")
            pieces.append(geom)
        if count:
            pieces.append("COUNT")
            pieces.append(count)
        if asc:
            pieces.append("ASC")
        if desc:
            pieces.append("DESC")
        if withdist:
            pieces.append("WITHDIST")
        if withoutwkts:
            pieces.append("WITHOUTWKT")

        return self.execute_command("GIS.SEARCH", *pieces)

    def gis_del(self, area: KeyT, polygen_name) -> ResponseT:
        return self.execute_command("GIS.DEL", area, polygen_name)



import uuid

import pytest

from tair import ExgetResult, Tair, TairCluster
from tair.asyncio import Tair as AsyncTair
from tair.asyncio import TairCluster as AsyncTairCluster

from .conftest import (
    TAIR_CLUSTER_HOST,
    TAIR_CLUSTER_PASSWORD,
    TAIR_CLUSTER_PORT,
    TAIR_CLUSTER_SCHEME,
    TAIR_CLUSTER_USERNAME,
    TAIR_DB,
    TAIR_HOST,
    TAIR_PASSWORD,
    TAIR_PORT,
    TAIR_SCHEME,
    TAIR_USERNAME,
)


def test_from_url():
    url = f"{TAIR_SCHEME}://{TAIR_HOST}:{TAIR_PORT}/{TAIR_DB}"
    t = Tair.from_url(url, username=TAIR_USERNAME, password=TAIR_PASSWORD)
    key = "key_" + str(uuid.uuid4())
    value = "value_" + str(uuid.uuid4())

    assert t.exset(key, value)
    result: ExgetResult = t.exget(key)
    assert result.value == value.encode()
    assert result.version == 1

    t.close()


def test_from_url_cluster():
    url = f"{TAIR_CLUSTER_SCHEME}://{TAIR_CLUSTER_HOST}:{TAIR_CLUSTER_PORT}"
    tc = TairCluster.from_url(
        url, username=TAIR_CLUSTER_USERNAME, password=TAIR_CLUSTER_PASSWORD
    )
    key = "key_" + str(uuid.uuid4())
    value = "value_" + str(uuid.uuid4())

    assert tc.exset(key, value)
    result: ExgetResult = tc.exget(key)
    assert result.value == value.encode()
    assert result.version == 1

    tc.close()


@pytest.mark.asyncio
async def test_from_url_async():
    url = f"{TAIR_SCHEME}://{TAIR_HOST}:{TAIR_PORT}/{TAIR_DB}"
    t = AsyncTair.from_url(url, username=TAIR_USERNAME, password=TAIR_PASSWORD)
    key = "key_" + str(uuid.uuid4())
    value = "value_" + str(uuid.uuid4())

    assert await t.exset(key, value)
    result: ExgetResult = await t.exget(key)
    assert result.value == value.encode()
    assert result.version == 1

    await t.close()


@pytest.mark.asyncio
async def test_from_url_async_cluster():
    url = f"{TAIR_CLUSTER_SCHEME}://{TAIR_CLUSTER_HOST}:{TAIR_CLUSTER_PORT}"
    tc = AsyncTairCluster.from_url(
        url, username=TAIR_CLUSTER_USERNAME, password=TAIR_CLUSTER_PASSWORD
    )
    key = "key_" + str(uuid.uuid4())
    value = "value_" + str(uuid.uuid4())

    assert await tc.exset(key, value)
    result: ExgetResult = await tc.exget(key)
    assert result.value == value.encode()
    assert result.version == 1

    await tc.close()



#!/usr/bin/env python
from setuptools import setup

setup(
    name="tair",
    description="Python client for Tair",
    long_description=open("README.md").read().strip(),
    long_description_content_type="text/markdown",
    version="1.3.4",
    license="MIT",
    url="https://github.com/alibaba/tair-py",
    author="Vincil Lau",
    author_email="vincillau@outlook.com",
    python_requires=">=3.7",
    packages=["tair", "tair.asyncio"],
    install_requires=["redis >= 4.4.4"],
)



import json
import uuid

import pytest

from tair import ScandocidResult, Tair


class TestTairSearch:
    def test_tft_createindex(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""

        assert t.tft_createindex(index, mappings)
        t.delete(index)

    def test_tft_updateindex(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings1 = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        mappings2 = """
{
  "mappings": {
    "properties": { "product_group": { "type": "text", "analyzer": "chinese" } }
  }
}"""

        assert t.tft_createindex(index, mappings1)
        assert t.tft_updateindex(index, mappings2)
        t.delete(index)


    def test_tft_getindex(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""

        assert t.tft_getindex(index) is None
        assert t.tft_createindex(index, mappings)
        assert t.tft_getindex(index) is not None
        mappings1 = f"""
{{
  "{index}":{{
    "mappings":{{
      "_source":{{
        "enabled":true,
        "excludes":[
        ],
        "includes":[
        ]
      }},
      "dynamic":"false",
      "properties":{{
        "price":{{
          "boost":1.0,
          "enabled":true,
          "ignore_above":-1,
          "index":true,
          "similarity":"classic",
          "type":"double"
        }},
        "product_id":{{
          "boost":1.0,
          "enabled":true,
          "ignore_above":128,
          "index":true,
          "similarity":"classic",
          "type":"keyword"
        }},
        "product_name":{{
          "boost":1.0,
          "enabled":true,
          "ignore_above":-1,
          "index":true,
          "similarity":"classic",
          "type":"text"
        }},
        "product_title":{{
          "analyzer":"jieba",
          "boost":1.0,
          "enabled":true,
          "ignore_above":-1,
          "index":true,
          "similarity":"classic",
          "type":"text"
        }}
      }}
    }}
  }}
}}
"""
        result = t.tft_getindex_mappings(index)
        assert json.loads(mappings1) == json.loads(result)
        settings = f"""
{{
  "{index}":{{
    "settings":{{
    }}
  }}
}}
"""
        result = t.tft_getindex_settings(index)
        assert json.loads(settings) == json.loads(result)
        t.delete(index)

    def test_tft_adddoc(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document = '{"product_id":"product test"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_adddoc(index, document)
        t.delete(index)

    def test_tft_madddoc(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document1 = '{"product_id":"test1"}'
        document2 = '{"product_id":"test2"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_madddoc(index, {document1: "00001", document2: "00002"})
        t.delete(index)

    def test_tft_updatedocfield(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document1 = '{"product_id":"test1"}'
        document2 = '{"product_id":"test2"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document1, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_updatedocfield(index, "00001", document2)
        t.delete(index)

    def test_tft_deldocfield(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document = '{"product_id":"test1","product_name":"product test"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_deldocfield(index, "00001", ["product_name"]) == 1
        t.delete(index)

    def test_tft_incrlongdocfield(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "integer" }
    }
  }
}"""
        document = '{"product_id":"test1","price":100}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_incrlongdocfield(index, "00001", "price", 200) == 300
        t.delete(index)

    def test_tft_incrfloatdocfield(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document = '{"product_id":"test1","price":1.1}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_incrfloatdocfield(index, "00001", "price", 2.2) == pytest.approx(
            3.3
        )
        t.delete(index)

    def test_tft_getdoc(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document = '{"product_id":"test1","price":1.1}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_getdoc(index, "00002") is None
        assert (
            t.tft_getdoc(index, "00001")
            == '{"_id":"00001","_source":{"product_id":"test1","price":1.1}}'
        )
        t.delete(index)

    def test_tft_exists(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document = '{"product_id":"test1","price":1.1}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_exists(index, "00001") == 1
        assert t.tft_exists(index, "00002") == 0
        t.delete(index)

    def test_tft_docnum(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document1 = '{"product_id":"test1"}'
        document2 = '{"product_id":"test2"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_madddoc(index, {document1: "00001", document2: "00002"})
        assert t.tft_docnum(index) == 2
        t.delete(index)

    def test_tft_scandocid(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document1 = '{"product_id":"test1"}'
        document2 = '{"product_id":"test2"}'
        document3 = '{"product_id":"test3"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_madddoc(
            index, {document1: "00001", document2: "00002", document3: "00003"}
        )
        assert t.tft_scandocid(index, 0, count=3, match="*") == ScandocidResult(
            "0", ["00001", "00002", "00003"]
        )
        t.delete(index)

    def test_tft_deldoc(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document = '{"product_id":"test1","product_name":"product test"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_deldoc(index, {"00001", "00002"}) == 1
        t.delete(index)

    def test_tft_delall(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document = '{"product_id":"test1","product_name":"product test"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_adddoc(index, document, doc_id="00001") == '{"_id":"00001"}'
        assert t.tft_delall(index)
        t.delete(index)

    def test_tft_search(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document1 = '{"product_id":"test1"}'
        document2 = '{"product_id":"test2"}'
        document3 = '{"product_id":"test3"}'

        assert t.tft_createindex(index, mappings)
        assert t.tft_madddoc(
            index, {document1: "00001", document2: "00002", document3: "00003"}
        )

        want = f"""{{
  "hits": {{
    "hits": [
      {{
        "_id": "00001",
        "_index": "{index}",
        "_score": 1.0,
        "_source": {{ "product_id": "test1" }}
      }},
      {{
        "_id": "00002",
        "_index": "{index}",
        "_score": 1.0,
        "_source": {{ "product_id": "test2" }}
      }},
      {{
        "_id": "00003",
        "_index": "{index}",
        "_score": 1.0,
        "_source": {{ "product_id": "test3" }}
      }}
    ],
    "max_score": 1.0,
    "total": {{ "relation": "eq", "value": 3 }}
  }}
}}"""
        result = t.tft_search(index, '{"sort":[{"price":{"order":"desc"}}]}')
        assert json.loads(want) == json.loads(result)
        result = t.tft_search(index, '{"sort":[{"price":{"order":"desc"}}]}', True)
        assert json.loads(want) == json.loads(result)
        result = t.tft_explaincost(index, '{"sort":[{"price":{"order":"desc"}}]}')
        assert json.loads(result)['QUERY_COST']
        t.delete(index)

    def test_tft_msearch(self, t: Tair):
        index1 = "{idx}_" + str(uuid.uuid4())
        index2 = "{idx}_" + str(uuid.uuid4())
        mappings = """
{
  "mappings": {
    "_source": { "enabled": true },
    "properties": {
      "product_id": { "type": "keyword", "ignore_above": 128 },
      "product_name": { "type": "text" },
      "product_title": { "type": "text", "analyzer": "jieba" },
      "price": { "type": "double" }
    }
  }
}"""
        document1 = '{"product_id":"test1"}'
        document2 = '{"product_id":"test2"}'
        document3 = '{"product_id":"test3"}'
        document4 = '{"product_id":"test4"}'

        assert t.tft_createindex(index1, mappings)
        assert t.tft_createindex(index2, mappings)
        assert t.tft_madddoc(
            index1, {document1: "00001", document2: "00002"}
        )
        assert t.tft_madddoc(
            index2, {document3: "00003", document4: "00004"}
        )

        want = f"""{{
    "aux_info": {{"index_crc64": 5843875291690071373}},
    "hits": {{
        "hits": [
          {{
            "_id": "00001",
            "_index": "{index1}",
            "_score": 1.0,
            "_source": {{ "product_id": "test1" }}
          }},
          {{
            "_id": "00002",
            "_index": "{index1}",
            "_score": 1.0,
            "_source": {{ "product_id": "test2" }}
          }},
          {{
            "_id": "00003",
            "_index": "{index2}",
            "_score": 1.0,
            "_source": {{ "product_id": "test3" }}
          }},
          {{
            "_id": "00004",
            "_index": "{index2}",
            "_score": 1.0,
            "_source": {{ "product_id": "test4" }}
          }}
        ],
        "max_score": 1.0,
        "total": {{ "relation": "eq", "value": 4 }}
      }}
    }}"""
        result = t.tft_msearch(2, {index1, index2}, '{"sort":[{"_doc":{"order":"asc"}}]}')
        assert json.loads(want) == json.loads(result)
        t.delete(index1)
        t.delete(index2)

    def test_tft_analyzer(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())
        mappings = """
{
  "mappings":{
    "properties":{
      "f0":{
        "type":"text",
        "analyzer":"my_analyzer"
      }
    }
  },
  "settings":{
    "analysis":{
      "analyzer":{
        "my_analyzer":{
          "type":"standard"
        }
      }
    }
  }
}"""
        text = 'This is tair-py.'

        assert t.tft_createindex(index, mappings)
        assert t.tft_analyzer("standard", text) == t.tft_analyzer("my_analyzer", text, index)
        assert 'consuming time' in str(t.tft_analyzer("standard", text, None, True))

        t.delete(index)

    def test_tft_addsug(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())

        assert (
            t.tft_addsug(index, {"redis is a memory database": 3, "redis cluster": 10})
            == 2
        )
        t.delete(index)

    def test_tft_delsug(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())

        assert (
            t.tft_addsug(index, {"redis is a memory database": 3, "redis cluster": 10})
            == 2
        )
        assert t.tft_delsug(index, ("redis is a memory database", "redis cluster")) == 2
        t.delete(index)

    def test_tft_sugnum(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())

        assert (
            t.tft_addsug(index, {"redis is a memory database": 3, "redis cluster": 10})
            == 2
        )
        assert t.tft_sugnum(index) == 2
        t.delete(index)

    def test_tft_getsug(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())

        assert (
            t.tft_addsug(index, {"redis is a memory database": 3, "redis cluster": 10})
            == 2
        )
        assert sorted(t.tft_getsug(index, "res", max_count=2, fuzzy=True)) == [
            "redis cluster",
            "redis is a memory database",
        ]
        t.delete(index)

    def test_tft_getallsugs(self, t: Tair):
        index = "idx_" + str(uuid.uuid4())

        assert (
            t.tft_addsug(index, {"redis is a memory database": 3, "redis cluster": 10})
            == 2
        )
        assert sorted(t.tft_getallsugs(index)) == [
            "redis cluster",
            "redis is a memory database",
        ]
        t.delete(index)

    def test_scandocid_result_eq(self):
        assert ScandocidResult("0", ["00001", "00002", "00003"]) == ScandocidResult(
            "0", ["00001", "00002", "00003"]
        )
        assert not ScandocidResult("0", ["00001", "00002", "00003"]) == ScandocidResult(
            "0", ["00001", "00002"]
        )
        assert not ScandocidResult("0", ["00001", "00002", "00003"]) == 1

    def test_scandocid_result_ne(self):
        assert not ScandocidResult("0", ["00001", "00002", "00003"]) != ScandocidResult(
            "0", ["00001", "00002", "00003"]
        )
        assert ScandocidResult("0", ["00001", "00002", "00003"]) != ScandocidResult(
            "0", ["00001", "00002"]
        )
        assert ScandocidResult("0", ["00001", "00002", "00003"]) != 1

    def test_scandocid_result_repr(self):
        assert (
            str(ScandocidResult("10", ["00001", "00002", "00003"]))
            == f"{{cursor: 10, doc_ids: ['00001', '00002', '00003']}}"
        )



from tair.client import Tair
from tair.cluster import TairCluster
from tair.exceptions import (
    AuthenticationError,
    AuthenticationWrongNumberOfArgsError,
    BusyLoadingError,
    ChildDeadlockedError,
    ConnectionError,
    DataError,
    InvalidResponse,
    PubSubError,
    ReadOnlyError,
    ResponseError,
    TairError,
    TimeoutError,
    WatchError,
)
from tair.taircpc import CpcUpdate2judResult
from tair.tairgis import TairGisSearchMember, TairGisSearchRadius
from tair.tairhash import ExhscanResult, FieldValueItem, ValueVersionItem
from tair.tairroaring import TrScanResult
from tair.tairsearch import ScandocidResult
from tair.tairstring import ExcasResult, ExgetResult
from tair.tairts import Aggregation, TairTsSkeyItem
from tair.tairvector import TairVectorIndex, TairVectorScanResult
from tair.tairzset import TairZsetItem

__all__ = [
    "Aggregation",
    "CpcUpdate2judResult",
    "ExcasResult",
    "ExgetResult",
    "ExhscanResult",
    "FieldValueItem",
    "ScandocidResult",
    "Tair",
    "TairCluster",
    "TairGisSearchMember",
    "TairGisSearchRadius",
    "TairTsSkeyItem",
    "TairZsetItem",
    "TrScanResult",
    "ValueVersionItem",
    # errors
    "AuthenticationError",
    "AuthenticationWrongNumberOfArgsError",
    "BusyLoadingError",
    "ChildDeadlockedError",
    "ConnectionError",
    "DataError",
    "InvalidResponse",
    "PubSubError",
    "ReadOnlyError",
    "ResponseError",
    "TairError",
    "TimeoutError",
    "WatchError",
    "TairVectorScanResult",
    "TairVectorIndex",
]



import uuid

from tair import Tair


class TestTairBloom:
    def test_bf_reserve(self, t: Tair):
        key = "key_" + str(uuid.uuid4())

        assert t.bf_reserve(key, 0.01, 100)

    def test_bf_add(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        item = "item_" + str(uuid.uuid4())

        assert t.bf_reserve(key, 0.01, 100)
        assert t.bf_add(key, item) == 1

    def test_bf_add_maybe_exist(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        item = "item_" + str(uuid.uuid4())

        assert t.bf_reserve(key, 0.01, 100)
        assert t.bf_add(key, item) == 1
        assert t.bf_add(key, item) == 0

    def test_bf_madd(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        item1 = "item_" + str(uuid.uuid4())
        item2 = "item_" + str(uuid.uuid4())
        item3 = "item_" + str(uuid.uuid4())

        assert t.bf_reserve(key, 0.01, 100)
        assert t.bf_add(key, item2) == 1
        assert t.bf_madd(key, (item1, item2, item3)) == [1, 0, 1]

    def test_bf_exists(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        item = "item_" + str(uuid.uuid4())

        assert t.bf_reserve(key, 0.01, 100)
        assert t.bf_exists(key, item) == 0
        assert t.bf_add(key, item) == 1
        assert t.bf_exists(key, item) == 1

    def test_bf_mexists(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        item1 = "item_" + str(uuid.uuid4())
        item2 = "item_" + str(uuid.uuid4())
        item3 = "item_" + str(uuid.uuid4())

        assert t.bf_reserve(key, 0.01, 100)
        assert t.bf_mexists(key, (item1, item2, item3)) == [0, 0, 0]
        assert t.bf_madd(key, (item1, item2, item3)) == [1, 1, 1]
        assert t.bf_mexists(key, (item1, item2, item3)) == [1, 1, 1]

    def test_bf_insert(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        item1 = "item_" + str(uuid.uuid4())
        item2 = "item_" + str(uuid.uuid4())
        item3 = "item_" + str(uuid.uuid4())

        assert t.bf_insert(key, [item1, item2, item3], 100, 0.01) == [1, 1, 1]
        assert t.bf_insert(key, [item1, item2, item3], 100, 0.01) == [0, 0, 0]

    def test_bf_insert_nocreate(self, t: Tair):
        key = "key_" + str(uuid.uuid4())
        item1 = "item_" + str(uuid.uuid4())
        item2 = "item_" + str(uuid.uuid4())
        item3 = "item_" + str(uuid.uuid4())

        t.bf_reserve(key, 0.01, 100)
        assert t.bf_insert(key, [item1, item2, item3], nocreate=True) == [1, 1, 1]
